
#include "inc.h"

#include "lib.h"
#include "minix.h"
#include "iwram_arm.h"
#include "bios_arm.h"
#include "div_arm.h"

#include "main.h"

//BG FontgꍇARgAEg폜
//#include "header\font\BGFont.h"


//Mode0
//BG2 \
//BG3 ͕
//Spr J[\
//---------------------------------------------------------------------------
#define STATE_INIT 				1
#define STATE_RESET				2
#define STATE_INPUT				3
#define STATE_DRAW				5

#define SCREEN_SHOW_BG			0x01
#define SCREEN_SHOW_LEFT		0x02
#define SCREEN_SHOW_RIGHT		0x04
#define SCREEN_SHOW_CENTER		0x08

//---------------------------------------------------------------------------

typedef struct {
	//show flag
	u8  showFlag;
	//buffer
	u16 buf [SCREEN_CX  ][SCREEN_CY];
	u16 chrL[SCREEN_CX/2][SCREEN_CY];
	u16 chrR[SCREEN_CX/2][SCREEN_CY];
	u16 chrC[SCREEN_CX/2][SCREEN_CY];
	//dummy
	u32 DummyU32 ALIGN(4);
} ST_SCREEN;

//---------------------------------------------------------------------------
ST_SCREEN Screen EWRAM_DATA;
ST_BG     BG2;
ST_BG     BG3;

u8  State;
u16 Key;
//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:  StateInit();  break;
			case STATE_RESET: StateReset(); break;
			case STATE_INPUT: StateInput(); break;
			case STATE_DRAW:  StateDraw();  break;

			default:
				DebugMessage("Err: Main State");
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitKey();
	InitIRQ();

	SetMode(MODE_3 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
	Key = 0x0000;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	REG_INTERUPT = (u32)IRQUserHandler;
	REG_IE       = IRQ_BIT_TIMER0;

	REG_TM0D     = 65536 - 2621;			// (((16*1024*1024) / 64) * 10) / 1000
	REG_TM0CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	ScreenInit();
	ScreenSetData();

	State = STATE_INPUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenInit()
{
	_Memset((u8*)&Screen, 0x00, sizeof(ST_SCREEN));
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenSetData()
{
	ST_SCREEN* p = &Screen;

//	LZ77UnCompWram((u16*)bg_backBitmap,   p->buf);
	LZ77UnCompWram((u16*)bg_leftBitmap,   p->chrL);
	LZ77UnCompWram((u16*)bg_rightBitmap,  p->chrR);
	LZ77UnCompWram((u16*)bg_centerBitmap, p->chrC);
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInput()
{
	WaitForVsync();
	Key = KeyGet();

	if( (Key == KEY_NONE) || (Key & KEY_DEC) )
	{
		return;
	}

	ST_SCREEN* p = &Screen;

	if(Key & KEY_UP   ) p->showFlag ^= SCREEN_SHOW_BG;
	if(Key & KEY_LEFT ) p->showFlag ^= SCREEN_SHOW_LEFT;
	if(Key & KEY_RIGHT) p->showFlag ^= SCREEN_SHOW_RIGHT;
	if(Key & KEY_DOWN ) p->showFlag ^= SCREEN_SHOW_CENTER;

	State = STATE_DRAW;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateDraw()
{
	ST_SCREEN* p = &Screen;

	if(p->showFlag & SCREEN_SHOW_BG)
	{
		ScreenBufCopyBg();
	}
	else
	{
		ScreenBufCls();
	}

	if(p->showFlag & SCREEN_SHOW_LEFT)
	{
		ScreenBufCopyLeft();
	}

	if(p->showFlag & SCREEN_SHOW_RIGHT)
	{
		ScreenBufCopyRight();
	}

	if(p->showFlag & SCREEN_SHOW_CENTER)
	{
		ScreenBufCopyCenter();
	}

	ScreenVRAMCopyBuf();
	State = STATE_INPUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenVRAMCopyBuf()
{
	WaitForVsync();

	REG_DM3SAD = (u32)&Screen.buf;
	REG_DM3DAD = (u32)VideoBuffer;
	REG_DM3CNT = (240 * 160)
	             | ( (DMA_SIZE_16 | DMA_TRANSFER_ON | DMA_TIMING_NOW | DMA_REPEAT_OFF |
	                  DMA_SAD_INC | DMA_DAD_INC     | DMA_INTR_OFF) << 16 );
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCls()
{
	REG_DM3SAD = (u32)&Screen.DummyU32;
	REG_DM3DAD = (u32)&Screen.buf;
	REG_DM3CNT = (240 * 160)
	             | ( (DMA_SIZE_16 | DMA_TRANSFER_ON | DMA_TIMING_NOW | DMA_REPEAT_OFF |
	                  DMA_SAD_FIX | DMA_DAD_INC     | DMA_INTR_OFF) << 16 );
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCopyBg()
{
	LZ77UnCompWram((u16*)bg_backBitmap, Screen.buf);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCopyRight()
{
	ScreenBufFillCopy(120, 0, 120, 160, (u16*)Screen.chrR);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCopyLeft()
{
	ScreenBufFillCopy(0, 0, 120, 160, (u16*)Screen.chrL);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCopyCenter()
{
	ScreenBufFillCopy(60, 0, 120, 160, (u16*)Screen.chrC);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufFillCopy(u16 sx, u16 sy, u16 cx, u16 cy, u16* img)
{
	u16* scr = (u16*)Screen.buf + (sy * SCREEN_CX) + sx;
	u16  x,y;

	for(y=0; y<cy; y++)
	{
		for(x=0; x<cx; x++)
		{
			if( img[y*cx + x] == RGB(0,31,0) )
			{
				continue;
			}

			scr[y*SCREEN_CX + x] = img[y*cx + x];
		}
	}
}
